#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#define MAX_LEN 20
#define LETTER_WIDTH 5
#include <util/delay.h>
#include <math.h>

char first_row[MAX_LEN] = "MATRIX";
int first_row_len = 6;

const int letters[][5] = 
{ 
 {0x7e, 0x09, 0x09, 0x09, 0x7e}, //A
 {0x7f, 0x49, 0x49, 0x49, 0x36}, //B
 {0x3e, 0x41, 0x41, 0x41, 0x22}, //C
 {0x7f, 0x41, 0x41, 0x22, 0x1c}, //D
 {0x7f, 0x49, 0x49, 0x49, 0x49}, //E
 {0x7f, 0x09, 0x09, 0x09, 0x09}, //F
 {0x7e, 0x41, 0x49, 0x49, 0x38}, //G
 {0x7f, 0x08, 0x08, 0x08, 0x7f}, //H
 {0x00, 0x41, 0x7f, 0x41, 0x00}, //I
 {0x30, 0x40, 0x41, 0x7f, 0x01}, //J
 {0x7f, 0x08, 0x14, 0x22, 0x41}, //K
 {0x7f, 0x40, 0x40, 0x40, 0x40}, //L
 {0x7f, 0x02, 0x04, 0x02, 0x7f}, //M
 {0x7f, 0x02, 0x0c, 0x10, 0x7f}, //N
 {0x3e, 0x41, 0x41, 0x41, 0x3e}, //O
 {0x7f, 0x49, 0x49, 0x49, 0x36}, //B
 {0x1e, 0x21, 0x21, 0x21, 0x5e}, //Q
 {0x7f, 0x09, 0x19, 0x29, 0x46}, //R
 {0x26, 0x49, 0x49, 0x49, 0x32}, //S
 {0x01, 0x01, 0x7f, 0x01, 0x01}, //T
 {0x3f, 0x40, 0x40, 0x40, 0x3f}, //U
 {0x07, 0x38, 0x40, 0x38, 0x07}, //V
 {0x3f, 0x40, 0x3f, 0x40, 0x3f}, //W
 {0x63, 0x14, 0x08, 0x14, 0x63}, //X
 {0x07, 0x08, 0x70, 0x08, 0x07}, //Y
 {0x61, 0x51, 0x49, 0x45, 0x43}, //Z  
 {0x00, 0x00, 0x00, 0x00, 0x00} // space
};

volatile unsigned char textArray[512];
volatile unsigned int arrLen = 0;
volatile unsigned int arrIx = 0;

void buildFirstRow();
void buildSndRow();
void incrSegment();
void initTimers();

int main(){
	int i;
	int seqNr = 0;
	sei();
	arrIx = 0;
	arrLen = 0;
	DDRA = 0XFF;
	DDRC = 0xFF;
	buildFirstRow();
	initTimers();
	/* Flashy animation: spiral led */
	while(1){
		if (seqNr == 0)
			for (i = 0; i < 8; i++){
				PORTC |= (1 << i); // led on
				_delay_ms(500);
				PORTC &= (1 << i); // led off
			}
		if (seqNr == 1)
			for (i = 0; i < 8; i++){
				PORTC |= (1 << i); // led on
				PORTC |= (1 << (8 - i - 1)); // led on
				_delay_ms(500);
				PORTC &= (1 << i); // led off
				PORTC &= (1 << (8 - i - 1)); // led off
			}
	}
	return 0;
}


void buildFirstRow(void){
	unsigned int i, j;
	for(i = 0; i < first_row_len; i++){
		for(j = 0; j < LETTER_WIDTH ; j++){
			textArray[arrLen] = letters[first_row[i] - 'A'][j];
			arrLen = arrLen + 1;
		}
		for(j = 0; j < LETTER_WIDTH ; j++){
			textArray[arrLen] = 0;
			arrLen = arrLen + 1;
		}
	}
	for(i = 0; i < first_row_len; i++){
		for(j = 0; j < LETTER_WIDTH; j++){
			textArray[arrLen] = 0;
			arrLen = arrLen + 1;
		}
	}
}

void incrSegment(void){
	if(arrIx < arrLen){
		PORTA = 255 ^ textArray[arrIx++];
	} else {
		arrIx = 0;
	}
}
ISR(SIG_OUTPUT_COMPARE1A){
	incrSegment();
}

void initTimers(){
	// initializeaza timer pentru intreruperi la intervale de 0.01s
	OCR1A = 13;
	/* Initializare Timer 1 */
	TCCR1A = 0;
	TCCR1B = (1<<WGM12) | _BV(CS12) | _BV(CS10);
	
	/* Setare intrerupere */
	TIMSK1 |= _BV(OCIE1A);
}
